/**
 * Ein Knoten einer Schlange aus Piktogrammen.
 * 
 * @author Bernd Gramlich 
 * @version 2012-11-09
 */
public class Knoten extends Schlangenelement
{
    /*
     * Inhalt des Knotens ist ein Piktogramm.
     */
    private Person ich;

    /*
     * Nachfolger des Knotens ist ein Schlangenelement,
     * also entweder wieder ein Knoten oder der Abschluss
     * der Warteschlange.
     */
    private Schlangenelement nachfolger;

    /**
     * Konstruktor.
     * Erzeuge gem den Parametern einen neuen Knoten mit Nachfolger
     * "meinNachfolger" und zeichne sein Piktogramm ab den Koordinaten (x|y).
     */
    public Knoten (Schlangenelement meinNachfolger, int x, int y,
        boolean weiblich, String name, String hautfarbe, String haarfarbe,
        String schuhfarbe, String kleiderfarbe1, String kleiderfarbe2,
        String kleiderfarbe3)
    {
        nachfolger = meinNachfolger;

        if (weiblich)
        {
            ich = new Frau (3, x, y, name, hautfarbe, haarfarbe, schuhfarbe,
                kleiderfarbe1, kleiderfarbe2, kleiderfarbe3);
        }
        else
        {   
            ich = new Mann (3, x, y, name, hautfarbe, haarfarbe, schuhfarbe,
                kleiderfarbe1, kleiderfarbe2, kleiderfarbe3);
        }
    }

    /**
     * Nenne den Nachfolger des Knotens.
     */
    public Schlangenelement gibNachfolger ()
    {
        return nachfolger;
    }

    /**
     * Nenne den Inhalt des Knotens.
     */
    public Person gibInhalt ()
    {
        return ich;
    }

    /**
     * Verschaffe dem Knoten einen neuen Nachfolger.
     */
    public void setzeNachfolger (Schlangenelement neuerNachfolger)
    {
        nachfolger = neuerNachfolger;
    }

    /**
     * Stelle hinten an die Schlange den neuen Kunden mit den
     * Daten an. Ich selbst bin der n-te Kunde.
     */
    public Schlangenelement stelleHintenAn (int n, boolean weiblich,
        String name, String hautfarbe, String haarfarbe, String schuhfarbe,
        String kleiderfarbe1, String kleiderfarbe2, String kleiderfarbe3)
    {
        /*
         * Ich berlasse meinem Nachfolger die Aufgabe, den neuen
         * Kunden hinten anzustellen. Wenn mein Nachfolger der 
         * Abschluss ist, dann tut er das auch tatschlich. Wenn
         * mein Nachfolger ein weiterer Knoten ist, dann wird er
         * den Befehl innerhalb der Schlange weiterreichen.
         * 
         * Mein Nachfolger ist dabei das (n+1)-te Schlangenelement.
         */
        nachfolger = nachfolger.stelleHintenAn (n+1, weiblich, name,
            hautfarbe, haarfarbe, schuhfarbe, kleiderfarbe1,
            kleiderfarbe2, kleiderfarbe3);
            
        return this;
    }

    /**
     * Lsche unter meinen Nachfolgern die erste Person namens
     * "name".
     */
    public Person loescheNachfolgerNamens (String name)
    {
        Person ausgabe;

        if (null != nachfolger.gibInhalt () 
            && nachfolger.gibInhalt ().gibName () == name)
        {
            /*
             * Wenn mein Nachfolger ein Knoten ist, der den
             * gesuchten Namen trgt, gib die Person aus und lsche
             * ihr Bild.
             */
            ausgabe = ((Knoten) nachfolger).ich;
            ausgabe.macheUnsichtbar ();

            /*
             * Mein neuer Nachfolger ist der Nachfolger meines
             * alten Nachfolgers.
             */
            nachfolger = nachfolger.gibNachfolger ();

            /*
             * Rcke die Schlange nach links auf.
             */
            nachfolger.verschiebeNachLinks ();
        }
        else
        {
            /*
             * Wenn mein Nachfolger nicht den gesuchten Namen
             * trgt, dann berlasse dem Nachfolger die
             * weitere Suche.
             */
            ausgabe = nachfolger.loescheNachfolgerNamens (name);
        }

        return ausgabe;
    }

    /**
     * Nenne die Lnge der Schlange von mir selbst ab.
     */
    public int gibLaenge ()
    {
        return 1 + nachfolger.gibLaenge ();
    }

    /**
     * Zhle alle Frauen von mir selbst ab.
     */
    public int gibFrauenzahl ()
    {
        if (ich.istWeiblich ())
        {
            return 1 + nachfolger.gibFrauenzahl ();
        }
        else
        {
            return nachfolger.gibFrauenzahl ();
        }
    }

    /**
     * Zhle alle Mnner von mir selbst ab.
     */
    public int gibMaennerzahl ()
    {
        if (ich.istWeiblich ())
        {
            return nachfolger.gibMaennerzahl ();
        }
        else
        {
            return 1 + nachfolger.gibMaennerzahl ();
        }
    }

    /**
     * Zhle alle Blondinen von mir selbst ab.
     */
    public int gibBlondinenzahl ()
    {
        if (ich.istWeiblich () && ich.gibHaarfarbe () == "blond")
        {
            return 1 + nachfolger.gibBlondinenzahl ();
        }
        else
        {
            return nachfolger.gibBlondinenzahl ();
        }
    }

    /**
     * Nenne die Anzahl der Buchstaben in allen Namen der Schlange
     * von mir selbst ab.
     */
    public int gibGesamtbuchstabenzahl ()
    {
        return ich.gibName ().length ()
        + nachfolger.gibGesamtbuchstabenzahl ();
    }

    /**
     * Nenne alle Namen in der Schlange von mir selbst ab.
     */
    public String nenneAlleNamen ()
    {
        return ich.gibName () + " "
        + nachfolger.nenneAlleNamen ();
    }

    /**
     * Zhle alle Anzugtrger von mir selbst ab.
     */
    public int gibAnzugtraegerzahl ()
    {
        if (!ich.istWeiblich () &&
            ich.gibKleiderfarbe1 () == ich.gibKleiderfarbe3 ())
        {
            return 1 + nachfolger.gibAnzugtraegerzahl ();
        }
        else
        {
            return nachfolger.gibAnzugtraegerzahl ();
        }
    }

    /**
     * Zhle alle Kleidtrgerinnen von mir selbst ab.
     */
    public int gibKleidtraegerinnenzahl ()
    {
        if (ich.istWeiblich () &&
            ich.gibKleiderfarbe1 () == ich.gibKleiderfarbe2 ())
        {
            return 1 + nachfolger.gibKleidtraegerinnenzahl ();
        }
        else
        {
            return nachfolger.gibKleidtraegerinnenzahl ();
        }
    }

    /**
     * Zhle alle Leute mit Anfangsbuchstabe L von mir selbst ab.
     */
    public int zaehleAnfangsbuchstabeL ()
    {
        if (ich.gibName ().substring (0, 1).equals ("L"))
        {
            return 1 + nachfolger.zaehleAnfangsbuchstabeL ();
        }
        else
        {
            return nachfolger.zaehleAnfangsbuchstabeL ();
        }
    }

    /**
     * Verkrze meinen und alle weitere Namen in der Schlange
     * auf einen dreibuchstabigen Spitznamen.
     */
    public void verkuerzeNamen ()
    {
        /*
         * Verkrze meinen Namen auf seine ersten drei Buchstaben.
         */
        if (ich.gibName ().length () > 3)
        {
            ich.setzeName (ich.gibName ().substring (0, 3));
        }

        /*
         * Fordere meinen Nachfolger auf, seinen Namen ebenfalls
         * zu verkrzen.
         */
        nachfolger.verkuerzeNamen ();
    }

    /**
     * Verschiebe den Knoten und alle seine Nachfolger um
     * 100 Bildpunkte nach links.
     */
    public void verschiebeNachLinks ()
    {
        /*
         * Ich verschiebe mein eigenes Bild um 100 Punkte nach
         * links.
         */
        ich.verschiebe (-100, 0);

        /*
         * Ich fordere meinen Nachfolger auf, sich ebenfalls nach
         * links zu verschieben.
         */
        nachfolger.verschiebeNachLinks ();
    }

    /**
     * Verschiebe den Knoten und alle seine Nachfolger um
     * 100 Bildpunkte nach rechts.
     */
    public void verschiebeNachRechts ()
    {
        /*
         * Ich fordere meinen Nachfolger auf, sich ebenfalls nach
         * links zu verschieben.
         */
        nachfolger.verschiebeNachRechts ();

        /*
         * Ich verschiebe mein eigenes Bild um 100 Punkte nach
         * rechts.
         */
        ich.verschiebe (100, 0);
    }

    /**
     * Fge einen schwarzen Peter unmittelbar vor meinem Nachfolger
     * namens "name" ein. Ich selbst bin der n-te Kunde.
     */
    public void draengleVorNachfolgerNamens (String name, int n)
    {
        Schlangenelement alterNachfolger;

        if (null != nachfolger.gibInhalt () 
            && nachfolger.gibInhalt ().gibName () == name)
        {
            /*
             * Wenn mein Nachfolger ein Knoten ist und den
             * gesuchten Namen trgt, merke ich ihn mir als meinen
             * alten Nachfolger.
             */
            alterNachfolger = nachfolger;

            /*
             * Alle meine Nachfolger mssen nach rechts aufrcken.
             */
            nachfolger.verschiebeNachRechts ();

            /*
             * Mein neuer Nachfolger ist der schwarze Peter. Sein
             * Nachfolger ist mein alter Nachfolger.
             */
            nachfolger = new Knoten (alterNachfolger, n * 100, 30,
                false, "Peter", "schwarz", "schwarz", "schwarz",
                "schwarz", "schwarz", "schwarz");
        }
        else
        {
            /*
             * Wenn mein Nachfolger nicht den gesuchten Namen
             * trgt, dann berlasse ich ihm die
             * weitere Suche.
             */
            nachfolger.draengleVorNachfolgerNamens (name, n+1);
        }
    }
}
