/**
 * Eine Warteschlange aus Piktogrammen.
 * Realisiert mit einer Liste von Knoten.
 * 
 * @author Bernd Gramlich 
 * @version 2012-10-27
 */
public class Schlange
{
    private Schlangenelement erstesSchlangenelement;

    /**
     * Konstruktor.
     * Erzeuge eine leere Schlange. Das ist also eine
     * Schlange, die nur aus einem Abschluss besteht.
     */
    public Schlange()
    {
        erstesSchlangenelement = new Abschluss ();
    }

    /**
     * Nchster Kunde.
     * Nimm den vordersten Kunde aus der Schlange und rcke auf.
     * Gib den Kunden als Rckgabewert.
     * Falls die Schlange leer ist, gib null zurck.
     */
    public Person naechsterKunde ()
    {
        /*
         * Wir geben am Ende die erste Person in der Schlange aus,
         * wenn sie existiert.
         */
        Person ausgabe;

        /*
         * Falls niemand in der Schlange steht ...
         */
        if (erstesSchlangenelement instanceof Abschluss)
        {
            /*
             * ... gib null aus ...
             */
            ausgabe = null;
        }
        /*
         * ... ansonsten ...
         */
        else
        {
            /*
             * ... gib die Person im ersten Knoten aus.
             */
            ausgabe = erstesSchlangenelement.gibInhalt ();
            ausgabe.macheUnsichtbar ();

            /*
             * Der Nachfolger des ersten Knotens wird neuer erster
             * Knoten. 
             */
            erstesSchlangenelement = erstesSchlangenelement.gibNachfolger ();

            /*
             * Lasse die Schlange nach links aufrcken.
             */
            verschiebeNachLinks ();
        }

        return ausgabe;
    }

    /**
     * Neuer Kunde.
     * Fge gem den Parametern einen neuen Kunden an die Schlange an.
     */
    public void neuerKunde (boolean weiblich, String name,
        String hautfarbe, String haarfarbe, String schuhfarbe,
        String kleiderfarbe1, String kleiderfarbe2, String kleiderfarbe3)
    {
        erstesSchlangenelement =
            erstesSchlangenelement.stelleHintenAn (1, weiblich, name,
            hautfarbe, haarfarbe, schuhfarbe, kleiderfarbe1,
            kleiderfarbe2, kleiderfarbe3);
    }

    /**
     * Entferne.
     * Lsche den ersten Kunden mit dem Namen "name" aus der Schlange.
     * Falls kein Kunde dieses Namens ansteht, gib null zurck.
     */
    public Person loesche (String name)
    {
        if (null != erstesSchlangenelement.gibInhalt () 
            && erstesSchlangenelement.gibInhalt ().gibName () == name)
        {
            /*
             * Wenn der erste Kunde den Namen "name" trgt, verlsst
             * es auf normale Weise die Schlange.
             */
            return naechsterKunde ();
        }
        else
        {
            /*
             * Andernfalls suchen wir unter den Nachfolgern des
             * ersten Knotens nach einem Kunden namens "name".
             */
            return erstesSchlangenelement.loescheNachfolgerNamens (name);
        }
    }

    /**
     * Lnge.
     * Nenne die Lnge der Schlange.
     */
    public int gibLaenge ()
    {
        return erstesSchlangenelement.gibLaenge ();
    }

    /**
     * Zufallskunden.
     * Fge n neue Zufallskunden an das Ende der Schlange an.
     */
    public void neueZufallskunden (int n)
    {
        boolean weiblich;
        String name;
        String hautfarbe;
        String haarfarbe;
        String schuhfarbe;
        String kleiderfarbe1;
        String kleiderfarbe2;
        String kleiderfarbe3;

        int zufall;

        for (int i = 1; i <= n; i++)
        {
            zufall = (int) Math.floor (4 * Math.random ());

            switch (zufall)
            {
                case 0:  weiblich = false; break;
                default: weiblich = true; break;
            }

            zufall = (int) Math.floor (10 * Math.random ());

            if (weiblich)
            {
                switch (zufall)
                {
                    case 0:  name = "Mia"; break;
                    case 1:  name = "Emma"; break;
                    case 2:  name = "Hanna"; break;
                    case 3:  name = "Anna"; break;
                    case 4:  name = "Lea"; break;
                    case 5:  name = "Leonie"; break;
                    case 6:  name = "Lina"; break;
                    case 7:  name = "Marie"; break;
                    case 8:  name = "Sophia"; break;
                    default: name = "Lena"; break;
                }
            }
            else
            {
                switch (zufall)
                {
                    case 0:  name = "Ben"; break;
                    case 1:  name = "Leon"; break;
                    case 2:  name = "Lucas"; break;
                    case 3:  name = "Finn"; break;
                    case 4:  name = "Jonas"; break;
                    case 5:  name = "Maximilian"; break;
                    case 6:  name = "Louis"; break;
                    case 7:  name = "Paul"; break;
                    case 8:  name = "Felix"; break;
                    default: name = "Luca"; break;
                }
            }

            zufall = (int) Math.floor (10 * Math.random ());

            switch (zufall)
            {
                case 0:  hautfarbe = "gebrunt"; break;
                case 1:  hautfarbe = "hellbraun"; break;
                default: hautfarbe = "bleich"; break;
            }

            zufall = (int) Math.floor (10 * Math.random ());

            switch (zufall)
            {
                case 0:  haarfarbe = "rot"; break;
                case 1:
                case 2:
                case 3:  haarfarbe = "blond"; break;
                case 4:
                case 5:
                case 6:  haarfarbe = "braun"; break;
                default: haarfarbe = "schwarz"; break;
            }

            zufall = (int) Math.floor (10 * Math.random ());

            if (weiblich)
            {
                switch (zufall)
                {
                    case 0:
                    case 1:  kleiderfarbe1 = "rot"; break;
                    case 2:
                    case 3:  kleiderfarbe1 = "pink"; break;
                    case 4:
                    case 5:  kleiderfarbe1 = "lila"; break;
                    case 6:  kleiderfarbe1 = "gelb"; break;
                    case 7:  kleiderfarbe1 = "orange"; break;
                    case 8:  kleiderfarbe1 = "trkis"; break;
                    default: kleiderfarbe1 = "braun"; break;
                }
            }
            else
            {
                switch (zufall)
                {
                    case 0:
                    case 1: 
                    case 2:  kleiderfarbe1 = "grau"; break;
                    case 3: 
                    case 4:  kleiderfarbe1 = "braun"; break;
                    case 5:
                    case 6:  kleiderfarbe1 = "dunkelbraun"; break;
                    case 7: 
                    case 8:  kleiderfarbe1 = "schwarz"; break;
                    default: kleiderfarbe1 = "blau"; break;
                }
            }

            zufall = (int) Math.floor (10 * Math.random ());

            if (weiblich)
            {
                switch (zufall)
                {
                    case 0: 
                    case 1:  kleiderfarbe3 = "grau"; break;
                    case 2:
                    case 3:  kleiderfarbe3 = "hellbraun"; break;
                    case 4:
                    case 5:  kleiderfarbe3 = "schwarz"; break;
                    case 6:  kleiderfarbe3 = kleiderfarbe1; break;
                    default: kleiderfarbe3 = hautfarbe; break;
                }
            }
            else
            {
                switch (zufall)
                {
                    case 0:  kleiderfarbe3 = "grau"; break;
                    case 1:  kleiderfarbe3 = "braun"; break;
                    case 2:  kleiderfarbe3 = "dunkelbraun"; break;
                    case 3:  kleiderfarbe3 = "schwarz"; break;
                    case 4:
                    case 5:
                    case 6:  kleiderfarbe3 = "blau"; break;
                    default: kleiderfarbe3 = kleiderfarbe1;
                }
            }

            zufall = (int) Math.floor (10 * Math.random ());

            if (weiblich)
            {
                switch (zufall)
                {
                    case 0:  kleiderfarbe2 = "rot"; break;
                    case 1:  kleiderfarbe2 = "pink"; break;
                    case 2:  kleiderfarbe2 = "lila"; break;
                    case 3:  kleiderfarbe2 = "gelb"; break;
                    case 4:  kleiderfarbe2 = "trkis"; break;
                    default: kleiderfarbe2 = kleiderfarbe1; break;
                }
            }
            else
            {
                switch (zufall)
                {
                    case 0:  kleiderfarbe2 = "rot"; break;
                    case 1:  kleiderfarbe2 = "pink"; break;
                    case 2:  kleiderfarbe2 = "orange"; break;
                    case 3:  kleiderfarbe2 = "grn"; break;
                    case 4:  kleiderfarbe2 = "trkis"; break;
                    case 5:  kleiderfarbe2 = "blau"; break;
                    case 6:  kleiderfarbe2 = "lila"; break;
                    case 7:  kleiderfarbe2 = "grau"; break;
                    case 8:  kleiderfarbe2 = "braun"; break;
                    default: kleiderfarbe2 = "schwarz"; break;
                }
            }

            zufall = (int) Math.floor (10 * Math.random ());

            if (weiblich)
            {
                switch (zufall)
                {
                    case 0:  schuhfarbe = "schwarz"; break;
                    case 1:  
                    case 2:  schuhfarbe = "hellbraun"; break;
                    case 3:  
                    case 4:  schuhfarbe = "dunkelbraun"; break;
                    case 5:
                    case 6:
                    case 7:  schuhfarbe = "braun"; break;
                    default: schuhfarbe = kleiderfarbe1; break;
                }
            }
            else
            {
                switch (zufall)
                {
                    case 0:
                    case 1:  schuhfarbe = "schwarz"; break;
                    case 2: 
                    case 3:  schuhfarbe = "hellbraun"; break;
                    case 4:
                    case 5:  schuhfarbe = "dunkelbraun"; break;
                    default: schuhfarbe = "braun"; break;
                }
            }

            neuerKunde (weiblich, name, hautfarbe, haarfarbe, schuhfarbe, 
                kleiderfarbe1, kleiderfarbe2, kleiderfarbe3);
        }
    }

    /**
     * Nenne die Anzahl der Frauen in der Warteschlange.
     */
    public int gibFrauenzahl ()
    {
        return erstesSchlangenelement.gibFrauenzahl ();
    }

    /**
     * Nenne die Anzahl der Mnner in der Warteschlange.
     */
    public int gibMaennerzahl ()
    {
        return erstesSchlangenelement.gibMaennerzahl ();
    }

    /**
     * Nenne die Anzahl der Blondinen in der Warteschlange.
     */
    public int gibBlondinenzahl ()
    {
        return erstesSchlangenelement.gibBlondinenzahl ();
    }

    /**
     * Nenne die Anzahl der Buchstaben in allen Namen der Schlange.
     */
    public int gibGesamtbuchstabenzahl ()
    {
        return erstesSchlangenelement.gibGesamtbuchstabenzahl ();
    }

    /**
     * Nenne alle Namen der Personen in der Schlange hintereinander.
     */
    public String nenneAlleNamen ()
    {
        return erstesSchlangenelement.nenneAlleNamen ();
    }

    /**
     * Nenne die Anzahl der Anzugtrger in der Schlange.
     */
    public int gibAnzugtraegerzahl ()
    {
        return erstesSchlangenelement.gibAnzugtraegerzahl ();
    }

    /**
     * Nenne die Anzahl der Kleidtrgerinnen in der Schlange.
     */
    public int gibKleidtraegerinnenzahl ()
    {
        return erstesSchlangenelement.gibKleidtraegerinnenzahl ();
    }

    /**
     * Bestimme die Anzahl der Personen mit Anfangsbuchstabe L.
     */
    public int zaehleAnfangsbuchstabeL ()
    {
        return erstesSchlangenelement.zaehleAnfangsbuchstabeL ();
    }

    /**
     * Verkuerze alle Namen in der Schlange auf einen
     * dreibuchstabigen Spitznamen.
     */
    public void verkuerzeNamen ()
    {
        erstesSchlangenelement.verkuerzeNamen ();
    }

    /**
     * Verschiebe die Schlange um 100 Bildpunkte nach rechts.
     */
    private void verschiebeNachRechts ()
    {
        erstesSchlangenelement.verschiebeNachRechts ();
    }

    /**
     * Verschiebe die Schlange um 100 Bildpunkte nach links.
     */
    private void verschiebeNachLinks ()
    {
        erstesSchlangenelement.verschiebeNachLinks ();
    }

    /**
     * Fge einen schwarzen Peter unmittelbar vor dem ersten
     * Kunden namens "name" ein.
     */
    public void draengleVor (String name)
    {
        Schlangenelement altesErstesSchlangenelement;

        if (null != erstesSchlangenelement.gibInhalt () 
            && erstesSchlangenelement.gibInhalt ().gibName () == name)
        {
            /*
             * Wenn das erste Schlangenelement ein Knoten ist,
             * der den gesuchten Namen trgt, merke es Dir als
             * altes erstes Schlangenelement.
             */
            altesErstesSchlangenelement = erstesSchlangenelement;

            /*
             * Rcke die gesamte Schlange nach rechts.
             */
            verschiebeNachRechts ();

            /*
             * Mache den schwarzen Peter zum neuen ersten
             * Schlangenelement mit dem alten ersten Schlangenelement
             * als Nachfolger.
             */
            erstesSchlangenelement = new Knoten (altesErstesSchlangenelement,
                 0, 30, false, "Peter", "schwarz", "schwarz", "schwarz",
                 "schwarz", "schwarz", "schwarz");
        }
        else
        {
            /*
             * Wenn der erste Knoten nicht den gesuchten Namen
             * trgt, dann suche unter seinen Nachfolgern nach
             * einer Gelegenheit zum Vordrngeln..
             */
            erstesSchlangenelement.draengleVorNachfolgerNamens (name, 1);
        }
    }
}